@@ -96,27 +96,24 @@ where
 {
     cx.group_use_children(
         |_| Rc::new(RefCell::new(RenderFlex::new())),
+        |_| false,
         content,
         |node, children| {
             let mut flex = node.borrow_mut();
             flex.children.clear();
-            for child in children {
-                if let Some(c) = child.downcast_ref::<Rc<RefCell<RenderLabel>>>().cloned() {
+            for child in children.iter() {
+                if let Some(c) = child.cast_ref::<Rc<RefCell<RenderLabel>>>().cloned() {
                     flex.children.push(c);
-                } else if let Some(c) = child.downcast_ref::<Rc<RefCell<RenderImage>>>().cloned() {
+                } else if let Some(c) = child.cast_ref::<Rc<RefCell<RenderImage>>>().cloned() {
                     flex.children.push(c);
-                } else if let Some(c) = child.downcast_ref::<Rc<RefCell<RenderFlex>>>().cloned() {
+                } else if let Some(c) = child.cast_ref::<Rc<RefCell<RenderFlex>>>().cloned() {
                     flex.children.push(c);
-                } else if let Some(c) = child
-                    .downcast_ref::<Rc<RefCell<dyn RenderObject>>>()
-                    .cloned()
-                {
+                } else if let Some(c) = child.cast_ref::<Rc<RefCell<dyn RenderObject>>>().cloned() {
                     flex.children.push(c);
                 }
             }
         },
-        |_| false,
-        |_| {},
+        |_, _| {},
         |_| {},
     );
 }
