@@ -14,6 +14,18 @@ pub trait ComposeNode: Debug + Downcast {}
 impl_downcast!(ComposeNode);
 impl<T: Debug + Downcast> ComposeNode for T {}
 
+impl dyn ComposeNode {
+    #[inline]
+    pub fn cast_ref<T: 'static>(&self) -> Option<&T> {
+        self.as_any().downcast_ref::<T>()
+    }
+
+    #[inline]
+    pub fn cast_mut<T: 'static>(&mut self) -> Option<&mut T> {
+        self.as_any_mut().downcast_mut::<T>()
+    }
+}
+
 type Tape = Vec<Slot<Box<dyn ComposeNode>>>;
 
 #[derive(Debug, Clone, Copy)]
@@ -108,7 +120,7 @@ impl Composer {
 
         if let Some((p_slot_id, p_size, p_data)) = cached {
             if curr_slot_id == p_slot_id {
-                if let Some(node) = p_data.as_any().downcast_ref::<Node>() {
+                if let Some(node) = p_data.cast_ref::<Node>() {
                     if log_enabled!(Trace) {
                         trace!(
                             "{}:{}{} | {:?} | {:?}",
@@ -181,7 +193,7 @@ impl Composer {
         F: FnOnce() -> Node,
         Node: ComposeNode + Clone,
     {
-        self.memo(|_| factory(), |_| true, |_| {}, |n| n.clone())
+        self.group(|_| factory(), |_| true, |_| {}, |_, _| {}, |n| n.clone())
     }
 
     #[track_caller]
@@ -199,121 +211,70 @@ impl Composer {
         U: FnOnce(&mut Node),
         O: FnOnce(&Node) -> Output,
     {
-        self.group(factory, |_| {}, skip, update, output)
+        self.group(factory, skip, |_| {}, |n, _| update(n), output)
     }
 
     #[track_caller]
-    pub fn group<F, Node, C, S, U, O, Output>(
+    pub fn group<F, Node, C, CO, S, U, O, Output>(
         &mut self,
         factory: F,
-        children: C,
         skip: S,
+        content: C,
         update: U,
         output: O,
     ) -> Output
     where
         F: FnOnce(&mut Composer) -> Node,
         Node: ComposeNode,
-        C: FnOnce(&mut Composer),
+        C: FnOnce(&mut Composer) -> CO,
         S: FnOnce(&mut Node) -> bool,
-        U: FnOnce(&mut Node),
+        U: FnOnce(&mut Node, CO),
         O: FnOnce(&Node) -> Output,
     {
-        self.slot(
-            factory,
-            children,
-            |_, _| {},
-            false,
-            |_, _| {},
-            skip,
-            update,
-            output,
-        )
+        self.slot(factory, skip, content, false, |_, _| {}, update, output)
     }
 
     #[track_caller]
-    pub fn group_apply_children<F, Node, C, Children, AC, S, U, O, Output>(
+    pub fn group_use_children<F, Node, C, CO, UC, S, U, O, Output>(
         &mut self,
         factory: F,
-        children: C,
-        apply_children: AC,
         skip: S,
-        update: U,
-        output: O,
-    ) -> Output
-    where
-        F: FnOnce(&mut Composer) -> Node,
-        Node: ComposeNode,
-        C: FnOnce(&mut Composer) -> Children,
-        AC: FnOnce(&mut Node, Children),
-        S: FnOnce(&mut Node) -> bool,
-        U: FnOnce(&mut Node),
-        O: FnOnce(&Node) -> Output,
-    {
-        self.slot(
-            factory,
-            children,
-            apply_children,
-            false,
-            |_, _| {},
-            skip,
-            update,
-            output,
-        )
-    }
-
-    #[track_caller]
-    pub fn group_use_children<F, Node, C, UC, S, U, O, Output>(
-        &mut self,
-        factory: F,
-        children: C,
+        content: C,
         use_children: UC,
-        skip: S,
         update: U,
         output: O,
     ) -> Output
     where
         F: FnOnce(&mut Composer) -> Node,
         Node: ComposeNode,
-        C: FnOnce(&mut Composer),
-        UC: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         S: FnOnce(&mut Node) -> bool,
-        U: FnOnce(&mut Node),
+        C: FnOnce(&mut Composer) -> CO,
+        UC: FnOnce(&mut Node, Children),
+        U: FnOnce(&mut Node, CO),
         O: FnOnce(&Node) -> Output,
     {
-        self.slot(
-            factory,
-            children,
-            |_, _| {},
-            true,
-            use_children,
-            skip,
-            update,
-            output,
-        )
+        self.slot(factory, skip, content, true, use_children, update, output)
     }
 
     #[track_caller]
     #[allow(clippy::too_many_arguments)]
-    pub fn slot<F, Node, C, Children, AC, UC, S, U, O, Output>(
+    pub fn slot<F, Node, C, CO, UC, S, U, O, Output>(
         &mut self,
         factory: F,
-        children: C,
-        apply_children: AC,
+        skip: S,
+        content: C,
         require_use_children: bool,
         use_children: UC,
-        skip: S,
         update: U,
         output: O,
     ) -> Output
     where
         F: FnOnce(&mut Composer) -> Node,
         Node: ComposeNode,
-        C: FnOnce(&mut Composer) -> Children,
-        AC: FnOnce(&mut Node, Children),
-        UC: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         S: FnOnce(&mut Node) -> bool,
-        U: FnOnce(&mut Node),
+        C: FnOnce(&mut Composer) -> CO,
+        UC: FnOnce(&mut Node, Children),
+        U: FnOnce(&mut Node, CO),
         O: FnOnce(&Node) -> Output,
     {
         // remember current slot states
@@ -353,8 +314,7 @@ impl Composer {
             .get_mut(curr_cursor)
             .map(|slot| {
                 if curr_slot_id == slot.id {
-                    let slot_data = slot.data.as_mut();
-                    if let Some(node) = slot_data.as_any_mut().downcast_mut::<Node>() {
+                    if let Some(node) = slot.data.as_mut().cast_mut::<Node>() {
                         (true, true, skip(node))
                     } else {
                         (true, false, false)
@@ -402,10 +362,10 @@ impl Composer {
             }
 
             let slot = self.tape.get_mut(curr_cursor).expect("slot");
-            let slot_data = slot.data.as_mut();
-            let node = slot_data
-                .as_any_mut()
-                .downcast_mut::<Node>()
+            let node = slot
+                .data
+                .as_mut()
+                .cast_mut::<Node>()
                 .expect("downcast node");
 
             // skip to slot end
@@ -442,7 +402,7 @@ impl Composer {
         }
 
         // new or update case
-        let c = children(self);
+        let co = content(self);
         assert!(
             id == self.id && self.composing && self.tape.len() > curr_cursor,
             "Composer is in inconsistent state"
@@ -452,27 +412,25 @@ impl Composer {
         let (head_slots, tail_slots) = self.tape.split_at_mut(child_start);
 
         let slot = head_slots.last_mut().expect("slot");
-        let slot_data = slot.data.as_mut();
-        let node = slot_data
-            .as_any_mut()
-            .downcast_mut::<Node>()
+        let node = slot
+            .data
+            .as_mut()
+            .cast_mut::<Node>()
             .expect("downcast node");
 
-        apply_children(node, c);
-        if require_use_children {
-            let child_depth = curr_depth + 1;
-            let cn = self.slot_depth[child_start..self.cursor]
-                .iter()
-                .cloned()
-                .enumerate()
-                .filter_map(|(i, v)| if v == child_depth { Some(i) } else { None })
-                .filter_map(|c| tail_slots.get(c).map(|s| s.data.as_ref()))
-                .collect();
-            use_children(node, cn);
-        }
-        update(node);
         // update new slot size after children fn done
         slot.size = self.cursor - curr_cursor;
+
+        if require_use_children {
+            let node_children = Children {
+                tape: &mut tail_slots[..slot.size - 1],
+                slot_depth: &self.slot_depth[child_start..self.cursor],
+                depth: curr_depth + 1,
+            };
+            use_children(node, node_children);
+        }
+        update(node, co);
+
         if log_enabled!(Trace) && slot.size > 1 {
             trace!(
                 "{}{: >7}:{}{} | {:?} | {:?}",
@@ -488,3 +446,28 @@ impl Composer {
         output(node)
     }
 }
+
+#[derive(Debug)]
+pub struct Children<'a> {
+    pub(crate) tape: &'a mut [Slot<Box<dyn ComposeNode>>],
+    pub(crate) slot_depth: &'a [usize],
+    pub(crate) depth: usize,
+}
+
+impl<'a> Children<'a> {
+    pub fn iter(&self) -> impl Iterator<Item = &dyn ComposeNode> {
+        self.tape
+            .iter()
+            .zip(self.slot_depth)
+            .filter(|(_, d)| **d == self.depth)
+            .map(|(s, _)| s.data.as_ref())
+    }
+
+    pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut dyn ComposeNode> {
+        self.tape
+            .iter_mut()
+            .zip(self.slot_depth)
+            .filter(|(_, d)| **d == self.depth)
+            .map(|(s, _)| s.data.as_mut())
+    }
+}
